@@ -15,7 +15,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * Copyright (C) 2005 - 2008 Red Hat, Inc.
+ * Copyright (C) 2005 - 2010 Red Hat, Inc.
  * Copyright (C) 2006 - 2008 Novell, Inc.
  */
 
@@ -34,6 +34,8 @@
 #include <linux/if.h>
 #include <errno.h>
 
+#include <netlink/route/addr.h>
+
 #include "nm-glib-compat.h"
 #include "nm-device-ethernet.h"
 #include "nm-device-interface.h"
@@ -43,15 +45,14 @@
 #include "nm-supplicant-manager.h"
 #include "nm-supplicant-interface.h"
 #include "nm-supplicant-config.h"
-#include "nm-netlink.h"
 #include "nm-netlink-monitor.h"
-#include "NetworkManagerSystem.h"
+#include "nm-system.h"
 #include "nm-setting-connection.h"
 #include "nm-setting-wired.h"
 #include "nm-setting-8021x.h"
 #include "nm-setting-pppoe.h"
 #include "ppp-manager/nm-ppp-manager.h"
-#include "nm-utils.h"
+#include "nm-logging.h"
 #include "nm-properties-changed-signal.h"
 #include "nm-dhcp-manager.h"
 
@@ -105,7 +106,6 @@ typedef struct {
 
 	struct ether_addr	hw_addr;
 	gboolean			carrier;
-	guint32				ifindex;
 
 	NMNetlinkMonitor *  monitor;
 	gulong              link_connected_id;
@@ -133,7 +133,6 @@ enum {
 	PROP_HW_ADDRESS,
 	PROP_SPEED,
 	PROP_CARRIER,
-	PROP_IFINDEX,
 
 	LAST_PROP
 };
@@ -227,11 +226,11 @@ set_carrier (NMDeviceEthernet *self,
 	g_object_notify (G_OBJECT (self), NM_DEVICE_ETHERNET_CARRIER);
 
 	state = nm_device_interface_get_state (NM_DEVICE_INTERFACE (self));
-	nm_info ("(%s): carrier now %s (device state %d%s)",
-	         nm_device_get_iface (NM_DEVICE (self)),
-	         carrier ? "ON" : "OFF",
-	         state,
-	         defer_action ? ", deferring action for 4 seconds" : "");
+	nm_log_info (LOGD_HW | LOGD_ETHER, "(%s): carrier now %s (device state %d%s)",
+	             nm_device_get_iface (NM_DEVICE (self)),
+	             carrier ? "ON" : "OFF",
+	             state,
+	             defer_action ? ", deferring action for 4 seconds" : "");
 
 	if (defer_action)
 		priv->carrier_action_defer_id = g_timeout_add_seconds (4, carrier_action_defer_cb, self);
@@ -246,11 +245,10 @@ carrier_on (NMNetlinkMonitor *monitor,
 {
 	NMDevice *device = NM_DEVICE (user_data);
 	NMDeviceEthernet *self = NM_DEVICE_ETHERNET (device);
-	NMDeviceEthernetPrivate *priv = NM_DEVICE_ETHERNET_GET_PRIVATE (self);
 	guint32 caps;
 
 	/* Make sure signal is for us */
-	if (idx == priv->ifindex) {
+	if (idx == nm_device_get_ifindex (device)) {
 		/* Ignore spurious netlink messages */
 		caps = nm_device_get_capabilities (device);
 		if (!(caps & NM_DEVICE_CAP_CARRIER_DETECT))
@@ -267,11 +265,10 @@ carrier_off (NMNetlinkMonitor *monitor,
 {
 	NMDevice *device = NM_DEVICE (user_data);
 	NMDeviceEthernet *self = NM_DEVICE_ETHERNET (device);
-	NMDeviceEthernetPrivate *priv = NM_DEVICE_ETHERNET_GET_PRIVATE (self);
 	guint32 caps;
 
 	/* Make sure signal is for us */
-	if (idx == priv->ifindex) {
+	if (idx == nm_device_get_ifindex (device)) {
 		NMDeviceState state;
 		gboolean defer = FALSE;
 
@@ -311,6 +308,10 @@ constructor (GType type,
 	self = NM_DEVICE (object);
 	priv = NM_DEVICE_ETHERNET_GET_PRIVATE (self);
 
+	nm_log_dbg (LOGD_HW | LOGD_OLPC_MESH, "(%s): kernel ifindex %d",
+	            nm_device_get_iface (NM_DEVICE (self)),
+	            nm_device_get_ifindex (NM_DEVICE (self)));
+
 	caps = nm_device_get_capabilities (self);
 	if (caps & NM_DEVICE_CAP_CARRIER_DETECT) {
 		GError *error = NULL;
@@ -328,33 +329,39 @@ constructor (GType type,
 
 		/* Get initial link state */
 		if (!nm_netlink_monitor_get_flags_sync (priv->monitor,
-		                                        priv->ifindex,
+		                                        nm_device_get_ifindex (NM_DEVICE (self)),
 		                                        &ifflags,
 		                                        &error)) {
-			nm_warning ("couldn't get initial carrier state: (%d) %s",
-			            error ? error->code : -1,
-			            (error && error->message) ? error->message : "unknown");
+			nm_log_warn (LOGD_HW | LOGD_ETHER,
+			             "(%s): couldn't get initial carrier state: (%d) %s",
+			             nm_device_get_iface (NM_DEVICE (self)),
+			             error ? error->code : -1,
+			             (error && error->message) ? error->message : "unknown");
 			g_clear_error (&error);
 		} else
 			priv->carrier = !!(ifflags & IFF_LOWER_UP);
 
-		nm_info ("(%s): carrier is %s",
-		         nm_device_get_iface (NM_DEVICE (self)),
-		         priv->carrier ? "ON" : "OFF");
+		nm_log_info (LOGD_HW | LOGD_ETHER,
+		             "(%s): carrier is %s",
+		             nm_device_get_iface (NM_DEVICE (self)),
+		             priv->carrier ? "ON" : "OFF");
 
 		/* Request link state again just in case an error occurred getting the
 		 * initial link state.
 		 */
 		if (!nm_netlink_monitor_request_status (priv->monitor, &error)) {
-			nm_warning ("couldn't request carrier state: (%d) %s",
-			            error ? error->code : -1,
-			            (error && error->message) ? error->message : "unknown");
+			nm_log_warn (LOGD_HW | LOGD_ETHER,
+			             "(%s): couldn't request carrier state: (%d) %s",
+			             nm_device_get_iface (NM_DEVICE (self)),
+			             error ? error->code : -1,
+			             (error && error->message) ? error->message : "unknown");
 			g_clear_error (&error);
 		}
 	} else {
-		nm_info ("(%s): driver '%s' does not support carrier detection.",
-		         nm_device_get_iface (self),
-		         nm_device_get_driver (self));
+		nm_log_info (LOGD_HW | LOGD_ETHER,
+		             "(%s): driver '%s' does not support carrier detection.",
+		             nm_device_get_iface (self),
+		             nm_device_get_driver (self));
 		priv->carrier = TRUE;
 	}
 
@@ -417,8 +424,7 @@ real_hw_take_down (NMDevice *dev)
 NMDevice *
 nm_device_ethernet_new (const char *udi,
 						const char *iface,
-						const char *driver,
-						guint32 ifindex)
+						const char *driver)
 {
 	g_return_val_if_fail (udi != NULL, NULL);
 	g_return_val_if_fail (iface != NULL, NULL);
@@ -428,7 +434,6 @@ nm_device_ethernet_new (const char *udi,
 	                                  NM_DEVICE_INTERFACE_UDI, udi,
 	                                  NM_DEVICE_INTERFACE_IFACE, iface,
 	                                  NM_DEVICE_INTERFACE_DRIVER, driver,
-	                                  NM_DEVICE_ETHERNET_IFINDEX, ifindex,
 	                                  NM_DEVICE_INTERFACE_TYPE_DESC, "Ethernet",
 	                                  NM_DEVICE_INTERFACE_DEVICE_TYPE, NM_DEVICE_TYPE_ETHERNET,
 	                                  NULL);
@@ -450,14 +455,6 @@ nm_device_ethernet_get_address (NMDeviceEthernet *self, struct ether_addr *addr)
 	memcpy (addr, &(NM_DEVICE_ETHERNET_GET_PRIVATE (self)->hw_addr), sizeof (struct ether_addr));
 }
 
-guint32
-nm_device_ethernet_get_ifindex (NMDeviceEthernet *self)
-{
-	g_return_val_if_fail (self != NULL, FALSE);
-
-	return NM_DEVICE_ETHERNET_GET_PRIVATE (self)->ifindex;
-}
-
 /* Returns speed in Mb/s */
 static guint32
 nm_device_ethernet_get_speed (NMDeviceEthernet *self)
@@ -473,7 +470,7 @@ nm_device_ethernet_get_speed (NMDeviceEthernet *self)
 
 	fd = socket (PF_INET, SOCK_DGRAM, 0);
 	if (fd < 0) {
-		nm_warning ("couldn't open control socket.");
+		nm_log_warn (LOGD_HW, "couldn't open control socket.");
 		return 0;
 	}
 
@@ -508,15 +505,16 @@ real_update_hw_address (NMDevice *dev)
 
 	fd = socket (PF_INET, SOCK_DGRAM, 0);
 	if (fd < 0) {
-		nm_warning ("couldn't open control socket.");
+		nm_log_warn (LOGD_HW, "couldn't open control socket.");
 		return;
 	}
 
 	memset (&req, 0, sizeof (struct ifreq));
 	strncpy (req.ifr_name, nm_device_get_iface (dev), IFNAMSIZ);
 	if (ioctl (fd, SIOCGIFHWADDR, &req) < 0) {
-		nm_warning ("%s: (%s) error getting hardware address: %d",
-		            __func__, nm_device_get_iface (dev), errno);
+		nm_log_err (LOGD_HW | LOGD_ETHER,
+		            "(%s) error getting hardware address: %d",
+		            nm_device_get_iface (dev), errno);
 		goto out;
 	}
 
@@ -670,7 +668,8 @@ real_connection_secrets_updated (NMDevice *dev,
 		if (!strcmp (setting_name, NM_SETTING_802_1X_SETTING_NAME)) {
 			valid = TRUE;
 		} else {
-			nm_warning ("Ignoring updated secrets for setting '%s'.", setting_name);
+			nm_log_warn (LOGD_DEVICE, "Ignoring updated secrets for setting '%s'.",
+			             setting_name);
 		}
 	}
 
@@ -827,8 +826,10 @@ link_timeout_cb (gpointer user_data)
 	if (!setting_name)
 		goto time_out;
 
-	nm_info ("Activation (%s/wired): disconnected during authentication,"
-	         " asking for new key.", nm_device_get_iface (dev));
+	nm_log_info (LOGD_DEVICE | LOGD_ETHER,
+	             "Activation (%s/wired): disconnected during authentication,"
+	             " asking for new key.",
+	             nm_device_get_iface (dev));
 	supplicant_interface_release (self);
 
 	nm_device_state_changed (dev, NM_DEVICE_STATE_NEED_AUTH, NM_DEVICE_STATE_REASON_SUPPLICANT_DISCONNECT);
@@ -842,7 +843,8 @@ link_timeout_cb (gpointer user_data)
 	return FALSE;
 
 time_out:
-	nm_info ("%s: link timed out.", nm_device_get_iface (dev));
+	nm_log_warn (LOGD_DEVICE | LOGD_ETHER,
+	             "(%s): link timed out.", nm_device_get_iface (dev));
 	nm_device_state_changed (dev, NM_DEVICE_STATE_FAILED, NM_DEVICE_STATE_REASON_SUPPLICANT_DISCONNECT);
 
 	return FALSE;
@@ -863,7 +865,7 @@ schedule_state_handler (NMDeviceEthernet *self,
 
 	task = g_slice_new0 (SupplicantStateTask);
 	if (!task) {
-		nm_warning ("Not enough memory to process supplicant manager state change.");
+		nm_log_err (LOGD_DEVICE, "Not enough memory to process supplicant manager state change.");
 		return FALSE;
 	}
 
@@ -906,10 +908,11 @@ supplicant_mgr_state_cb (NMSupplicantInterface * iface,
                          guint32 old_state,
                          gpointer user_data)
 {
-	nm_info ("(%s): supplicant manager state:  %s -> %s",
-	         nm_device_get_iface (NM_DEVICE (user_data)),
-	         nm_supplicant_manager_state_to_string (old_state),
-	         nm_supplicant_manager_state_to_string (new_state));
+	nm_log_info (LOGD_DEVICE | LOGD_ETHER,
+	             "(%s): supplicant manager state:  %s -> %s",
+	             nm_device_get_iface (NM_DEVICE (user_data)),
+	             nm_supplicant_manager_state_to_string (old_state),
+	             nm_supplicant_manager_state_to_string (new_state));
 
 	schedule_state_handler (NM_DEVICE_ETHERNET (user_data),
 	                        supplicant_mgr_state_cb_handler,
@@ -936,7 +939,7 @@ build_supplicant_config (NMDeviceEthernet *self)
 
 	security = NM_SETTING_802_1X (nm_connection_get_setting (connection, NM_TYPE_SETTING_802_1X));
 	if (!nm_supplicant_config_add_setting_8021x (config, security, con_path, TRUE)) {
-		nm_warning ("Couldn't add 802.1X security setting to supplicant config.");
+		nm_log_warn (LOGD_DEVICE, "Couldn't add 802.1X security setting to supplicant config.");
 		g_object_unref (config);
 		config = NULL;
 	}
@@ -962,11 +965,17 @@ supplicant_iface_state_cb_handler (gpointer user_data)
 			success = nm_supplicant_interface_set_config (priv->supplicant.iface, config);
 			g_object_unref (config);
 
-			if (!success)
-				nm_warning ("Activation (%s/wired): couldn't send security "
-						  "configuration to the supplicant.", iface);
-		} else
-			nm_warning ("Activation (%s/wired): couldn't build security configuration.", iface);
+			if (!success) {
+				nm_log_err (LOGD_DEVICE | LOGD_ETHER,
+				            "Activation (%s/wired): couldn't send security "
+						    "configuration to the supplicant.",
+						    iface);
+			}
+		} else {
+			nm_log_warn (LOGD_DEVICE | LOGD_ETHER,
+			             "Activation (%s/wired): couldn't build security configuration.",
+			             iface);
+		}
 
 		if (!success)
 			nm_device_state_changed (device, NM_DEVICE_STATE_FAILED, NM_DEVICE_STATE_REASON_SUPPLICANT_CONFIG_FAILED);
@@ -990,10 +999,11 @@ supplicant_iface_state_cb (NMSupplicantInterface * iface,
                            gpointer user_data)
 {
 
-	nm_info ("(%s): supplicant interface state:  %s -> %s",
-	         nm_device_get_iface (NM_DEVICE (user_data)),
-	         nm_supplicant_interface_state_to_string (old_state),
-	         nm_supplicant_interface_state_to_string (new_state));
+	nm_log_info (LOGD_DEVICE | LOGD_ETHER,
+	             "(%s): supplicant interface state:  %s -> %s",
+	             nm_device_get_iface (NM_DEVICE (user_data)),
+	             nm_supplicant_interface_state_to_string (old_state),
+	             nm_supplicant_interface_state_to_string (new_state));
 
 	schedule_state_handler (NM_DEVICE_ETHERNET (user_data),
 	                        supplicant_iface_state_cb_handler,
@@ -1016,8 +1026,9 @@ supplicant_iface_connection_state_cb_handler (gpointer user_data)
 		 * schedule the next activation stage.
 		 */
 		if (nm_device_get_state (dev) == NM_DEVICE_STATE_CONFIG) {
-			nm_info ("Activation (%s/wired) Stage 2 of 5 (Device Configure) successful.",
-				    nm_device_get_iface (dev));
+			nm_log_info (LOGD_DEVICE | LOGD_ETHER,
+			             "Activation (%s/wired) Stage 2 of 5 (Device Configure) successful.",
+				         nm_device_get_iface (dev));
 			nm_device_activate_schedule_stage3_ip_config_start (dev);
 		}
 	} else if (task->new_state == NM_SUPPLICANT_INTERFACE_CON_STATE_DISCONNECTED) {
@@ -1040,10 +1051,11 @@ supplicant_iface_connection_state_cb (NMSupplicantInterface * iface,
                                       guint32 old_state,
                                       gpointer user_data)
 {
-	nm_info ("(%s) supplicant connection state:  %s -> %s",
-	         nm_device_get_iface (NM_DEVICE (user_data)),
-	         nm_supplicant_interface_connection_state_to_string (old_state),
-	         nm_supplicant_interface_connection_state_to_string (new_state));
+	nm_log_info (LOGD_DEVICE | LOGD_ETHER,
+	             "(%s) supplicant connection state:  %s -> %s",
+	             nm_device_get_iface (NM_DEVICE (user_data)),
+	             nm_supplicant_interface_connection_state_to_string (old_state),
+	             nm_supplicant_interface_connection_state_to_string (new_state));
 
 	schedule_state_handler (NM_DEVICE_ETHERNET (user_data),
 	                        supplicant_iface_connection_state_cb_handler,
@@ -1075,8 +1087,9 @@ supplicant_iface_connection_error_cb (NMSupplicantInterface *iface,
 	NMDeviceEthernetPrivate *priv = NM_DEVICE_ETHERNET_GET_PRIVATE (self);
 	guint id;
 
-	nm_info ("Activation (%s/wired): association request to the supplicant failed: %s - %s",
-	         nm_device_get_iface (NM_DEVICE (self)), name, message);
+	nm_log_warn (LOGD_DEVICE | LOGD_ETHER,
+	             "Activation (%s/wired): association request to the supplicant failed: %s - %s",
+	             nm_device_get_iface (NM_DEVICE (self)), name, message);
 
 	if (priv->supplicant.iface_con_error_cb_id)
 		g_source_remove (priv->supplicant.iface_con_error_cb_id);
@@ -1120,8 +1133,9 @@ handle_auth_or_fail (NMDeviceEthernet *self,
 		                            NULL);
 
 		g_object_set_data (G_OBJECT (connection), WIRED_SECRETS_TRIES, GUINT_TO_POINTER (++tries));
-	} else
-		nm_warning ("Cleared secrets, but setting didn't need any secrets.");
+	} else {
+		nm_log_info (LOGD_DEVICE, "Cleared secrets, but setting didn't need any secrets.");
+	}
 
 	return NM_ACT_STAGE_RETURN_POSTPONE;
 }
@@ -1140,15 +1154,17 @@ supplicant_connection_timeout_cb (gpointer user_data)
 	iface = nm_device_get_iface (device);
 
 	/* Authentication failed, encryption key is probably bad */
-	nm_info ("Activation (%s/wired): association took too long.", iface);
+	nm_log_warn (LOGD_DEVICE | LOGD_ETHER,
+	             "Activation (%s/wired): association took too long.", iface);
 
 	supplicant_interface_release (self);
 	req = nm_device_get_act_request (device);
 	g_assert (req);
 
-	if (handle_auth_or_fail (self, req, TRUE) == NM_ACT_STAGE_RETURN_POSTPONE)
-		nm_info ("Activation (%s/wired): asking for new secrets", iface);
-	else
+	if (handle_auth_or_fail (self, req, TRUE) == NM_ACT_STAGE_RETURN_POSTPONE) {
+		nm_log_info (LOGD_DEVICE | LOGD_ETHER,
+		             "Activation (%s/wired): asking for new secrets", iface);
+	} else
 		nm_device_state_changed (device, NM_DEVICE_STATE_FAILED, NM_DEVICE_STATE_REASON_NO_SECRETS);
 
 	return FALSE;
@@ -1165,7 +1181,9 @@ supplicant_interface_init (NMDeviceEthernet *self)
 	/* Create supplicant interface */
 	priv->supplicant.iface = nm_supplicant_manager_get_iface (priv->supplicant.mgr, iface, FALSE);
 	if (!priv->supplicant.iface) {
-		nm_warning ("Couldn't initialize supplicant interface for %s.", iface);
+		nm_log_err (LOGD_DEVICE | LOGD_ETHER,
+		            "Couldn't initialize supplicant interface for %s.",
+		            iface);
 		supplicant_interface_release (self);
 
 		return FALSE;
@@ -1213,7 +1231,7 @@ nm_8021x_stage2_config (NMDeviceEthernet *self, NMDeviceStateReason *reason)
 	connection = nm_act_request_get_connection (nm_device_get_act_request (NM_DEVICE (self)));
 	security = NM_SETTING_802_1X (nm_connection_get_setting (connection, NM_TYPE_SETTING_802_1X));
 	if (!security) {
-		nm_warning ("Invalid or missing 802.1X security");
+		nm_log_err (LOGD_DEVICE, "Invalid or missing 802.1X security");
 		*reason = NM_DEVICE_STATE_REASON_CONFIG_FAILED;
 		return ret;
 	}
@@ -1226,15 +1244,17 @@ nm_8021x_stage2_config (NMDeviceEthernet *self, NMDeviceStateReason *reason)
 	if (setting_name) {
 		NMActRequest *req = nm_device_get_act_request (NM_DEVICE (self));
 
-		nm_info ("Activation (%s/wired): connection '%s' has security, but secrets are required.",
-				 iface, nm_setting_connection_get_id (s_connection));
+		nm_log_info (LOGD_DEVICE | LOGD_ETHER,
+		             "Activation (%s/wired): connection '%s' has security, but secrets are required.",
+				     iface, nm_setting_connection_get_id (s_connection));
 
 		ret = handle_auth_or_fail (self, req, FALSE);
 		if (ret != NM_ACT_STAGE_RETURN_POSTPONE)
 			*reason = NM_DEVICE_STATE_REASON_NO_SECRETS;
 	} else {
-		nm_info ("Activation (%s/wired): connection '%s' requires no security. No secrets needed.",
-				 iface, nm_setting_connection_get_id (s_connection));
+		nm_log_info (LOGD_DEVICE | LOGD_ETHER,
+		             "Activation (%s/wired): connection '%s' requires no security. No secrets needed.",
+				     iface, nm_setting_connection_get_id (s_connection));
 
 		if (supplicant_interface_init (self))
 			ret = NM_ACT_STAGE_RETURN_POSTPONE;
@@ -1311,8 +1331,8 @@ pppoe_stage3_ip4_config_start (NMDeviceEthernet *self, NMDeviceStateReason *reas
 					   self);
 		ret = NM_ACT_STAGE_RETURN_POSTPONE;
 	} else {
-		nm_warning ("(%s): PPPoE failed to start: %s",
-		            nm_device_get_iface (NM_DEVICE (self)), err->message);
+		nm_log_warn (LOGD_DEVICE, "(%s): PPPoE failed to start: %s",
+		             nm_device_get_iface (NM_DEVICE (self)), err->message);
 		g_error_free (err);
 
 		g_object_unref (priv->ppp_manager);
@@ -1586,7 +1606,7 @@ ip4_match_config (NMDevice *self, NMConnection *connection)
 	int ifindex;
 	AddrData check_data;
 
-	ifindex = nm_device_ethernet_get_ifindex (NM_DEVICE_ETHERNET (self));
+	ifindex = nm_device_get_ifindex (self);
 
 	s_con = (NMSettingConnection *) nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION);
 	g_assert (s_con);
@@ -1608,9 +1628,9 @@ ip4_match_config (NMDevice *self, NMConnection *connection)
 
 	/* Get any saved leases that apply to this connection */
 	dhcp_mgr = nm_dhcp_manager_get ();
-	leases = nm_dhcp_manager_get_lease_ip4_config (dhcp_mgr,
-	                                               nm_device_get_iface (self),
-	                                               nm_setting_connection_get_uuid (s_con));
+	leases = nm_dhcp_manager_get_lease_config (dhcp_mgr,
+	                                           nm_device_get_iface (self),
+	                                           nm_setting_connection_get_uuid (s_con));
 	g_object_unref (dhcp_mgr);
 
 	method = nm_setting_ip4_config_get_method (s_ip4);
@@ -1758,9 +1778,6 @@ get_property (GObject *object, guint prop_id,
 	case PROP_CARRIER:
 		g_value_set_boolean (value, priv->carrier);
 		break;
-	case PROP_IFINDEX:
-		g_value_set_uint (value, priv->ifindex);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -1771,13 +1788,7 @@ static void
 set_property (GObject *object, guint prop_id,
 			  const GValue *value, GParamSpec *pspec)
 {
-	NMDeviceEthernetPrivate *priv = NM_DEVICE_ETHERNET_GET_PRIVATE (object);
-
 	switch (prop_id) {
-	case PROP_IFINDEX:
-		/* construct-only */
-		priv->ifindex = g_value_get_uint (value);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -1844,14 +1855,6 @@ nm_device_ethernet_class_init (NMDeviceEthernetClass *klass)
 							   FALSE,
 							   G_PARAM_READABLE));
 
-	g_object_class_install_property
-		(object_class, PROP_IFINDEX,
-		 g_param_spec_uint (NM_DEVICE_ETHERNET_IFINDEX,
-						   "Ifindex",
-						   "Interface index",
-						   0, G_MAXUINT32, 0,
-						   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | NM_PROPERTY_PARAM_NO_EXPORT));
-
 	/* Signals */
 	signals[PROPERTIES_CHANGED] = 
 		nm_properties_changed_signal_new (object_class,
@@ -1880,7 +1883,7 @@ supports_ethtool_carrier_detect (NMDeviceEthernet *self)
 
 	fd = socket (PF_INET, SOCK_DGRAM, 0);
 	if (fd < 0) {
-		nm_warning ("couldn't open control socket.");
+		nm_log_err (LOGD_HW, "couldn't open control socket.");
 		return FALSE;
 	}
 
@@ -1890,13 +1893,18 @@ supports_ethtool_carrier_detect (NMDeviceEthernet *self)
 	edata.cmd = ETHTOOL_GLINK;
 	ifr.ifr_data = (char *) &edata;
 
-	if (ioctl (fd, SIOCETHTOOL, &ifr) < 0)
+	errno = 0;
+	if (ioctl (fd, SIOCETHTOOL, &ifr) < 0) {
+		nm_log_dbg (LOGD_HW | LOGD_ETHER, "SIOCETHTOOL failed: %d", errno);
 		goto out;
+	}
 
 	supports_ethtool = TRUE;
 
 out:
 	close (fd);
+	nm_log_dbg (LOGD_HW | LOGD_ETHER, "ethtool %s supported",
+	            supports_ethtool ? "is" : "not");
 	return supports_ethtool;
 }
 
@@ -1920,8 +1928,13 @@ mdio_read (NMDeviceEthernet *self, int fd, struct ifreq *ifr, int location)
 	mii = (struct mii_ioctl_data *) &ifr->ifr_ifru;
 	mii->reg_num = location;
 
-	if (ioctl (fd, SIOCGMIIREG, ifr) == 0)
+	errno = 0;
+	if (ioctl (fd, SIOCGMIIREG, ifr) == 0) {
+		nm_log_dbg (LOGD_HW | LOGD_ETHER, "SIOCGMIIREG result 0x%X", mii->val_out);
 		val = mii->val_out;
+	} else {
+		nm_log_dbg (LOGD_HW | LOGD_ETHER, "SIOCGMIIREG failed: %d", errno);
+	}
 
 	return val;
 }
@@ -1937,19 +1950,24 @@ supports_mii_carrier_detect (NMDeviceEthernet *self)
 
 	fd = socket (PF_INET, SOCK_DGRAM, 0);
 	if (fd < 0) {
-		nm_warning ("couldn't open control socket.");
+		nm_log_err (LOGD_HW, "couldn't open control socket.");
 		return 0;
 	}
 
 	memset (&ifr, 0, sizeof (struct ifreq));
 	strncpy (ifr.ifr_name, nm_device_get_iface (NM_DEVICE (self)), IFNAMSIZ);
 
-	if (ioctl (fd, SIOCGMIIPHY, &ifr) < 0)
+	errno = 0;
+	if (ioctl (fd, SIOCGMIIPHY, &ifr) < 0) {
+		nm_log_dbg (LOGD_HW | LOGD_ETHER, "SIOCGMIIPHY failed: %d", errno);
 		goto out;
+	}
 
 	/* If we can read the BMSR register, we assume that the card supports MII link detection */
 	bmsr = mdio_read (self, fd, &ifr, MII_BMSR);
 	supports_mii = (bmsr != -1) ? TRUE : FALSE;
+	nm_log_dbg (LOGD_HW | LOGD_ETHER, "MII %s supported",
+	            supports_mii ? "is" : "not");
 
 out:
 	close (fd);
